from serpapi import GoogleSearch
import os
import json
import openai
import tenacity

api_key = os.getenv("SERPAPI_API_KEY")

model = "gpt-4-0613"

@tenacity.retry(    wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),    stop=tenacity.stop_after_attempt(5),    reraise=True, )
def return_code(model, user_input):
    
    
    response = openai.ChatCompletion.create(
        model=model,
        messages=[{"role": "system", "content": "You return search query for google based on user input. clarify and simplify the user input to match the search query for google"},
            {"role": "user", "content": f"{user_input}"}],
        # stream=True,
        functions=[
            {
                "name": "search_google",
                "description": "This function takes in 'search query' argument and retrives the search results from google",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "search_query" : {
                            "type": "string",
                            "description": "name of the search query based on user input",
                        },
                    },
                    "required": ["search_query"],
                    }
            }

        ],
        function_call={"name": "search_google"} # "auto" "none"
    )

    message = response.choices[0].message
    

    if message.get("function_call"):
        function_args = json.loads(message["function_call"]["arguments"]) 
        return function_args
    
    if message["content"]:
        print(colored(message["content"], "green"))

while True:

  user_input = input("Enter user input: ")
  function_args = return_code(model, user_input)



  q = function_args["search_query"]

  params = {
    "q": q,
    "location": "Austin, Texas, United States",
    "hl": "en",
    "gl": "us",
    "google_domain": "google.com",
    "api_key": api_key
  }

  # search only if the file does not exist
  if not os.path.exists("results.json"):
      search = GoogleSearch(params)
      # SAVE THE HTML FILESAVE THE RESULTS TO A FILE
      results = search.get_dict()
      print(results)
      with open('results.json', 'w') as outfile:
          json.dump(results, outfile)

  # read the first result from the file
  with open('results.json') as json_file:
      data = json.load(json_file)
      print(data['organic_results'][0]['title'])
      print(data['organic_results'][0]['link'])
      print(data['organic_results'][0]['snippet'])
      print(data['organic_results'][0]['cached_page_link'])
      print(data['organic_results'][0]['related_pages_link'])
      print(data['organic_results'][0]['sitelinks'])
      print(data['organic_results'][0]['position'])
